#include <stdio.h>
#include <stdlib.h>
#include "grass.h"

long int **imatrix(nrl, nrh, ncl, nch, error_msg)
long int nrl, nrh, ncl, nch;
char *error_msg;
/* Allocates a long int matrix with range [nrl ... nrh][ncl ... nch]
   Stolen from 'Numerical Recipes in C' */
{
   long int i;
   long int **m;

   /* Allocates pointers to rows */
   m = (long int **) malloc((unsigned)(nrh - nrl + 1)*sizeof(long int));
   if (!m)
   {
      *error_msg = ALLOCATION_ERROR;
      return(m);
   }
   m -= nrl;

   /* Allocate rows and set pointers to them. */
   for (i = nrl; i <= nrh; i++)
   {
      m[i]=(long int *) malloc((unsigned)(nch-ncl+1)*sizeof(long int));
      if (!m[i])
      {
	 *error_msg = ALLOCATION_ERROR;
	 return(m);
      }
      m[i] -= ncl;
   }

   /* Return pointer to array of pointer to rows. */
   *error_msg = OK;
   return(m);
}/* imatrix */


double *dvector(nl,nh, error_msg)
long int nl,nh;
char *error_msg;
{
   double *v;

   v=(double *)malloc((unsigned) (nh-nl+1)*sizeof(double));
   if (!v)
   {
      *error_msg = ALLOCATION_ERROR;
      return(v);
   }
   *error_msg = OK;
   return v-nl;
}/* dvector */


long int *ivector(nl,nh, error_msg)
long int nl,nh;
char *error_msg;
{
   long int *v;

   v=(long int *)malloc((unsigned) (nh-nl+1)*sizeof(long int));
   if (!v)
   {
      *error_msg = ALLOCATION_ERROR;
      return(v);
   }
   *error_msg = OK;
   return v-nl;
}/* ivector */


BIN_TYPE *binvector(nl,nh, error_msg)
long int nl,nh;
char *error_msg;
{
   BIN_TYPE *v;

   v=(BIN_TYPE *)malloc((unsigned) (nh-nl+1)*sizeof(BIN_TYPE));
   if (!v)
   {
      *error_msg = ALLOCATION_ERROR;
      return(v);
   }
   *error_msg = OK;
   return v-nl;
}/* binvector */


void free_imatrix(m, nrl, nrh, ncl, nch)
long int **m;
long int nrl, nrh, ncl, nch;
/* Frees a matrix allocated with imatrix */
{
   long int i;

   for (i=nrh; i >= nrl; i--)
      free((char*) (m[i] + ncl));
   free((char*) (m + nrl));
}/* free_imatrix */


void free_dvector(v,nl,nh)
double *v;
long int nl,nh;
{
   free((char*) (v+nl));
}/* free_dvector */


void free_ivector(v,nl,nh)
long int *v;
long int nl,nh;
{
   free((char*) (v+nl));
}/* free_ivector */

