#include "./include/header.h"
#include "./include/getinput.h"


/* Definitions */
#define VERSION 1.30
#define TRUE 1
#define FALSE 0
#ifndef PI
#define PI 3.14159265358979323846264338327950
#endif

/* Error Codes */
#define NO_ERROR 0
#define COMMAND_LINE_ERROR 1
#define OPEN_OUTPUT_FILE_ERROR 2
#define INVALID_OPTION 3
#define FILE_FORMAT_ERROR 4
#define FILE_ACCESS_ERROR 5
#define ALLOCATION_ERROR 6
#define NOT_SCALAR_DATA 9
#define M_OPTION_ERROR 10
#define E_OPTION_ERROR 11
#define R_OPTION_ERROR 12

/* Macros */

#define INT_POW(a,b,ans) ans=1.0; for(i=1;i<=b;i++) ans *= a
/* Raises a to the integer power of b and stores result in ans */

/**** Remember, var.l[0] for Sparcs, var.l[1] for Turbo C ****/
#define GET_BIASED_EXP(a,b) var.d = a; var.l[0] >>= 20; b = (var.l[0] & 2047)
/* Stores the biased exponent of 'a' into 'b' using the known structure
   of the IEEE format, so compatibility with machines other than the Sun
   Stations is not garaunteed.  A biased exponent is essentially 1023 plus
   the power of 2 that multiplies the mantissa of the number. */

/* Structure Definitions */

typedef struct
{
   long int num_pts;
   long int *point;
} BIN_TYPE;

/* Union Declarations */
union Var_Type
{
   double d;
   long int l[2];
};

/* Function Prototypes */

void get_parameters();                  /* Function in INPUT.C */
long int get_options();
void get_user_input();
void get_command_line_args();
void check_args();
void error();

void read_in_data();			/* Functions in PROCESS.C */
void compute_full_correlation_integral();
void compute_small_r_correlation_integral();
void write_output();
void determine_number_of_points();

long int **imatrix();			/* Functions in ALLOC.C */
double *dvector();
long int *ivector();
BIN_TYPE *binvector();
void free_imatrix();
void free_dvector();
void free_ivector();


/* Global Variables */
#ifndef VARS
#define VARS extern
#endif

VARS char input_file_name[NAME_LENGTH];
VARS char output_file_name[NAME_LENGTH];
VARS long int num_ref_pts;
VARS long int num_embedded_pts;
VARS long int min_dim;
VARS long int max_dim;
VARS long int time_delay;
VARS long int autocor_len;
VARS long int num_autocor_pts;
VARS long int trace;
VARS char INPUT_NUM_REF_POINTS; 	/* Flag to indicate computation
					   of averaged pointwise dim */
VARS char COMPUTE_FULL_CORR_INT;	/* Flag to indicate computation 
					   of correlation integral for all
					   r values as opposed to only
					   the small values */
VARS double max_dist;			/* Maximum r value of correlation
					   integral to be computed if
					   COMPUTE_FULL_CORR_INT == FALSE*/
VARS BIN_TYPE *bin;
VARS long int expansion_fact;		/* Indirectly controls number of 
					   points in output file */
VARS long int **bin_count;		/* Matrix to hold number of
					   distances in each bin */
VARS double *data;			/* Vector of data values */
VARS double max_val, min_val;		/* Extremums of Data values */
VARS union Var_Type var;		/* Used in determination of
					   exponent (GET_BIASED_EXP). */


