#include<stdio.h>
#include <string.h>
#include <stdlib.h>
#include "./include/getinput.h"

#define VARS
#define OK -1
#define COMMAND_LINE_ERROR 0
#define INVALID_OPTION 1
#define FILE_ACCESS_ERROR 2
#define FILE_FORMAT_ERROR 3
#define OPEN_OUTPUT_FILE_ERROR 4
#define ALLOCATION_ERROR 5

double get_slope();
void get_parameters();
long int get_options();
void check_args();
void error();
double *dvector();
void free_dvector();

/*Global Variable */
char input_file_name[80];
char output_file_name[80];
FILE *fd_in, *fd_out;

void main(argc, argv)
int argc;
char *argv[];
{
   int i;
   char line_buf[520];
   char *line_buf_ptr;
   double *C_prev, *C_cur, *C_next;
   double prev_log_r, cur_log_r, next_log_r;
   int num_dims;
   double slope;
   char error_msg;

   get_parameters(argc, argv);

   /*** Space over comments put in GRASS ***/
   do
   {
      fgets(line_buf, 520, fd_in);
   } while(line_buf[0] == '#');
   
   /*** Determine number of embedding dimensions in file ***/
   num_dims = 0;
   i = 0;
   while(i < strlen(line_buf))
   {
      if (line_buf[i] == ' ')
      { 
	 if (line_buf[i+1] == '\n') break;
	 else num_dims++;
      }
      i++;
   }

   C_prev = dvector((long)1, (long)num_dims, &error_msg);
   if (error_msg == ALLOCATION_ERROR)
      error(ALLOCATION_ERROR, "C_prev Vector");

   C_cur = dvector((long)1, (long)num_dims, &error_msg);
   if (error_msg == ALLOCATION_ERROR)
      error(ALLOCATION_ERROR, "C_cur Vector");

   C_next = dvector((long)1, (long)num_dims, &error_msg);
   if (error_msg == ALLOCATION_ERROR)
      error(ALLOCATION_ERROR, "C_next Vector");

   /** Read values off of first two lines ***/
   sscanf(line_buf, "%lf", &prev_log_r);
   line_buf_ptr = strchr(line_buf, ' ') + 1;

   for (i = 1; i <= num_dims; i++)
   {
      sscanf(line_buf_ptr, "%lf", &C_prev[i]);
      line_buf_ptr = strchr(line_buf_ptr, ' ') + 1;
   }

   fgets(line_buf, 520, fd_in);
   sscanf(line_buf, "%lf", &cur_log_r);
   line_buf_ptr = strchr(line_buf, ' ') + 1;

   for (i = 1; i <= num_dims; i++)
   {
      sscanf(line_buf_ptr, "%lf", &C_cur[i]);
      line_buf_ptr = strchr(line_buf_ptr, ' ') + 1;
   }

   do
   {
      if (fgets(line_buf, 520, fd_in) == NULL)
	 break;

      sscanf(line_buf, "%lf", &next_log_r);
      fprintf(fd_out, "%f ", cur_log_r);
      line_buf_ptr = strchr(line_buf, ' ') +1;

      for (i = 1; i <= num_dims; i++)
      {
	 sscanf(line_buf_ptr, "%lf", &C_next[i]);
	 line_buf_ptr = strchr(line_buf_ptr, ' ') + 1;
	 slope = get_slope(prev_log_r, cur_log_r, next_log_r,
			   C_prev[i], C_cur[i], C_next[i]);
	 fprintf(fd_out, "%f ", slope);
      }

      fprintf(fd_out, "\n");
      prev_log_r = cur_log_r;
      cur_log_r = next_log_r;
      for (i = 1; i <= num_dims; i++)
      {
	 C_prev[i] = C_cur[i];
	 C_cur[i] = C_next[i];
      }

   } while (1);

   fclose(fd_in);
   fclose(fd_out);
   free_dvector(C_prev, (long)1, (long)num_dims);
   free_dvector(C_cur, (long)1, (long)num_dims);

}/* main */

double get_slope(x0, x1, x2, F0, F1, F2)
double x0, x1, x2, F0, F1, F2;
/* Determines the best fit slope of three points */
{
  double num, den;

  num = 2.0 * F0 * x0 + 2.0 * F1 * x1 + 2.0 * F2 * x2
	-(F1 + F2)*x0 - (F0 + F2)*x1 - (F0 + F1)*x2;
  den = 2.0 * (x0 * x0 + x1 * x1 + x2 * x2
	     -x0 * x1 - x0 * x2 - x1 * x2);

  return(num/den);

}/* get_slope */

void get_parameters(argc, argv)
int argc;
char *argv[];
/* Gets parameters and options from either the command line or user
   input. */
{
   long int option_args;

   option_args = get_options(argc, argv);

   argc -= option_args;

   if (argc == 1) 
   {
      get_input("Enter Data File Name : ", input_file_name, NAME);
      get_input("Enter Output File Name : ", output_file_name, NAME);
   }

   else if (argc != 3)
      error(COMMAND_LINE_ERROR, NULL);

   else
   {
      sprintf(input_file_name, argv[++option_args]);
      sprintf(output_file_name, argv[++option_args]);
   }

   check_args();

}/* get_parameters */


long int get_options(argc, argv)
int argc;
char *argv[];
/* Checks for and processes options from command line.  Returns the number
   of arguments from command line that contained valid options. */
{
   long int i, n, len, arg_count = 1;
   long int num_option_args = 0;
   char buf[20];

   /* Set default option values. */

   if (argc == 1) return(0);

   while(arg_count < argc && argv[arg_count][0] == '-')
   {
      len = strlen(argv[arg_count]);
      for(n = 1; n < len; n++)
      {
	 switch(argv[arg_count][n])
	 {
	    default :
	       error(INVALID_OPTION);
	 }/* End switch statement */
      }
      arg_count++;
   }
   return(num_option_args);
}/* get_options */


void check_args()
/* Do a little error checking on arguments */
{
   char buf[80];

   if (!strcmp(input_file_name, output_file_name))
   {
      printf("Output File Name must be different from Data File Name\n");
      exit(-1);
   }
   
   /* Verify File. */
   if ((fd_in = fopen(input_file_name,"r")) == NULL)
      error(FILE_ACCESS_ERROR, input_file_name);

   fgets(buf, 520, fd_in);
   if (strncmp(buf, "#GRASS Output Results", 21))
      error(FILE_FORMAT_ERROR, input_file_name);

  /* Verify File Name. */
   if ((fd_out = fopen(output_file_name,"w")) == NULL)
      error(OPEN_OUTPUT_FILE_ERROR, output_file_name);
 
}/* check_args */


void error(error_type, error_buf)
char error_type;
char *error_buf;
/* Displays error messages and exits program if fatal */
{
   switch(error_type)
   {
      case COMMAND_LINE_ERROR :
      case INVALID_OPTION :
         if (error_type == COMMAND_LINE_ERROR)
            printf("Invalid Command Line. \n");
         if (error_type == INVALID_OPTION)
            printf("Invalid Option on Command Line. \n");
	 printf("Usage : grass_deriv input_file_name output_file_name\n");
	 exit(-1);
	 break;
      case FILE_ACCESS_ERROR :
	 printf("\tCannot access file %s.\n", error_buf);
	 exit(-1);
	 break;
      case FILE_FORMAT_ERROR :
	 printf("\tIt does not appear that %s was made be GRASS\n",
		error_buf);
         exit(-1);
      case OPEN_OUTPUT_FILE_ERROR :
	 printf("\tCannot Open writable file named %s", error_buf);
	 exit(-1);
      case ALLOCATION_ERROR :
	 printf("\tFatal error during allocation of %s\n", error_buf);
	 exit(-1);
   }
}/* error */

double *dvector(nl,nh, error_msg)
long int nl,nh;
char *error_msg;
{
   double *v;

   v=(double *)malloc((unsigned) (nh-nl+1)*sizeof(double));
   if (!v)
   {
      *error_msg = ALLOCATION_ERROR;
      return(v);
   }
   *error_msg = OK;
   return v-nl;
}/* dvector */

void free_dvector(v,nl,nh)
double *v;
long int nl,nh;
{
   free((char*) (v+nl));
}/* free_dvector */
