#include<stdio.h>
#include <string.h>
#include <stdlib.h>
#include "grass.h"


void get_parameters(argc, argv)
int argc;
char *argv[];
/* Gets parameters and options from either the command line or user
   input. */
{
   long int option_args;

   option_args = get_options(argc, argv);

   argc -= option_args;

   if (argc == 1) get_user_input();

   else if (argc != 7 && argc != 8)
      error(COMMAND_LINE_ERROR, NULL);

   else
     get_command_line_args(argc, argv, option_args);

   determine_number_of_points();

   check_args();

   if (trace >= 2)
   {
      /* Display all parameters */
      printf("\n%s's Contents : \n", input_file_name);
      printf("Attractor Name : %s\n", header.name);
      printf("Attractor Parameters : %s\n", header.pars);
      printf("Number of Dimensions of Attractor : %d\n", header.num_dim);
      printf("Initial Values : %s\n", header.inits);
      printf("Number of points in Data file according to header : %ld\n"
	      ,header.num_pts);
      printf("Number of points in transient : %ld\n"
	      ,header.tran_len);
      printf("\nExecution Parameters : \n");
      printf("Range of Embedding Dimensions : %ld to %ld\n", min_dim,
		max_dim);
      printf("Time Delay Value : %ld\n", time_delay);
      printf("Autocorrelation Length : %ld\n", autocor_len);
      printf("Number of points in largest embedding space : %ld\n",
	      num_embedded_pts);
      if (INPUT_NUM_REF_POINTS == TRUE)
      {
         printf("Computing Averaged Pointwise dimension with %ld reference "
	     ,num_ref_pts);
         printf("points \nused in calculation\n");
      }
      printf("Expansion Factor : %ld\n", expansion_fact);
      if (COMPUTE_FULL_CORR_INT == FALSE)
	 printf("Computing Correlation Integral up to r = %lg\n", 
		max_dist);

      printf("\nOutput File Name : %s\n", output_file_name);
   }
}/* get_parameters */


long int get_options(argc, argv)
int argc;
char *argv[];
/* Checks for and processes options from command line.  Returns the number
   of arguments from command line that contained valid options. */
{
   long int i, n, len, arg_count = 1;
   long int num_option_args = 0;
   char buf[20];

   /* Set default option values. */
   INPUT_NUM_REF_POINTS = FALSE;
   COMPUTE_FULL_CORR_INT = TRUE;
   expansion_fact = 2;

   if (argc == 1) return(0);

   while(arg_count < argc && argv[arg_count][0] == '-')
   {
      len = strlen(argv[arg_count]);
      for(n = 1; n < len; n++)
      {
	 switch(argv[arg_count][n])
	 {
	    case 'M' :
	       INPUT_NUM_REF_POINTS = TRUE;

	       /* Check that format is correct */
	       if (n == 1) num_option_args++;
	       if (argv[arg_count][n+1] != ':')
		  error(M_OPTION_ERROR);
	       i = n + 2;

	       /* Read off value. */
	       while(i < len)
	       {
		  buf[i-n-2] = argv[arg_count][i];
		  if (buf[i-n-2] < '0' || buf[i-n-2] > '9')
		     error(M_OPTION_ERROR);
		  i++;
	       }
	       buf[i-n-2] = 0;
	       num_ref_pts = atol(buf);
	       n = len;
	       break;
	    case 'E' :
	       /* Check that format is correct */
	       if (n == 1) num_option_args++;
	       if (argv[arg_count][n+1] != ':')
		  error(E_OPTION_ERROR);
	       i = n + 2;

	       /* Read off Value */
	       while(i < len)
	       {
		  buf[i-n-2] = argv[arg_count][i];
		  if (buf[i-n-2] < '0' || buf[i-n-2] > '9')
		     error(E_OPTION_ERROR);
		  i++;
	       }
	       buf[i-n-2] = 0;
	       expansion_fact = atol(buf);
	       n = len;
	       break;
	    case 'R' :
	       COMPUTE_FULL_CORR_INT = FALSE;

	       /* Check that format is correct */
	       if (n == 1) num_option_args++;
	       if (argv[arg_count][n+1] != ':')
		  error(R_OPTION_ERROR);
	       i = n + 2;

	       /* Read off Value */
	       while(i < len)
	       {
		  buf[i-n-2] = argv[arg_count][i];
		  if (!((buf[i-n-2] >= '0' && buf[i-n-2] <= '9') ||
		     buf[i-n-2] == '.' || buf[i-n-2] == 'e' ||
		     (buf[i-n-2] == '-' && buf[i-n-3] == 'e')))
		     error(R_OPTION_ERROR);
		  i++;
	       }
	       buf[i-n-2] = 0;
	       max_dist = atof(buf);
	       n = len;
	       break;
	    default :
	       error(INVALID_OPTION);
	 }/* End switch statement */
      }
      arg_count++;
   }
   return(num_option_args);
}/* get_options */


void get_user_input()
{
   char ans_buf[80];
   FILE *fd;
   char ans;

   get_input("Enter Data File Name : ", input_file_name, NAME);

   /* Verify File. */
   if ((fd = fopen(input_file_name,"r")) == NULL)
      error(FILE_ACCESS_ERROR, input_file_name);

   if (read_header(fd) == FILE_FORMAT_ERROR)
      error(FILE_FORMAT_ERROR, input_file_name);

   fclose(fd);

   get_input("Enter Output File Name : ", output_file_name, NAME);

   /* Verify File Name. */
   if ((fd = fopen(output_file_name,"w")) == NULL)
      error(OPEN_OUTPUT_FILE_ERROR, output_file_name);

   fclose(fd);

   printf("\nGRASS computes the correlation integral for a range of ");
   printf("embedding dimensions.\n");

   get_input("Enter the minimum Embedding Dimension : ", &min_dim, POS_INT);
   get_input("Enter the maximum Embedding Dimension : ", &max_dim, POS_INT);
   get_input("Enter time delay for embedding : ", &time_delay, POS_INT);
   get_input("Enter the autocorrelation length : ", &autocor_len, POS_INT);

   get_input("Do you want to limit the number of reference points (y/n) ? ",
	     &ans, YES_NO);
   if (ans == TRUE)
   {
      INPUT_NUM_REF_POINTS = TRUE;
      get_input("Enter Number of Reference Points : ", &num_ref_pts, POS_INT);
   }

   get_input("Do you want to change default number of output points (y/n) ? ",
	     &ans, YES_NO);
   if (ans == TRUE)
      get_input("Enter New Expansion Factor (default = 2) : ",
		&expansion_fact, POS_INT);

   if (INPUT_NUM_REF_POINTS == FALSE)
   {
      get_input("Do you want to limit the range of 'r' of Correlation Int (y/n)?",
		&ans, YES_NO);
      if (ans == TRUE)
      {
	 COMPUTE_FULL_CORR_INT = FALSE;
	 get_input("Enter maximum r value : ", &max_dist, POS_FLOAT);
      }
   }

   get_input("Enter trace level (0-2) : ", &trace, POS_INT);
   if (trace > 2) trace = 2;

}/* get_user_input */


void get_command_line_args(argc, argv, option_args)
int argc;
char *argv[];
long int option_args;
/* Grab options from command line */
{
   FILE *fd;

   sprintf(input_file_name, argv[++option_args]);

   /* Verify File. */
   if ((fd = fopen(input_file_name,"r")) == NULL)
      error(FILE_ACCESS_ERROR, input_file_name);

   if (read_header(fd) == FILE_FORMAT_ERROR)
      error(FILE_FORMAT_ERROR, input_file_name);

   fclose(fd);

   sprintf(output_file_name, argv[++option_args]);

   /* Verify File. */
   if ((fd = fopen(output_file_name,"w")) == NULL)
      error(OPEN_OUTPUT_FILE_ERROR, output_file_name);

   fclose(fd);

   min_dim = atol(argv[++option_args]);
   max_dim = atol(argv[++option_args]);
   time_delay = atol(argv[++option_args]);
   autocor_len = atol(argv[++option_args]);

   if (argc == 8)
   {
      trace = atoi(argv[++option_args]);
      if (trace < 0 || trace > 2) trace = 2;
   }
   else
      trace = 2;
}/* get_command_line_args */


void check_args()
/* Do a little error checking on arguments */
{
   long int i;

   if (header.num_dim != 1)
      error(NOT_SCALAR_DATA);

   if (num_ref_pts == 0)
   {
      printf("\tInvalid number of reference points.  Exiting Program.\n");
      exit(-1);
   }

   if (num_ref_pts > num_embedded_pts)
   {
      printf("\tThe Number of reference points cannot exceed the number\n");
      printf("\tof data points.  Exiting Program.\n");
      exit(-1);
   }

   if (min_dim > max_dim)
   {
      printf("\tInvalid range of embedding dimensions.\n");
      exit(-1);
   }
   if (expansion_fact == 0)
   {
      printf("\tExpansion factor must be greater than zero.\n");
      exit(-1);
   }
   if(time_delay == 0)
   {
      printf("\tTime delay value must be greater than zero.\n");
      exit(-1);
   }

   if (COMPUTE_FULL_CORR_INT == FALSE && INPUT_NUM_REF_POINTS == TRUE)
   {
      printf("\tYou cannot specify both the number of reference points and");
      printf(" the\n\tmaximum distance for Correlation Integral.  If you");
      printf(" want to compute\n\taveraged pointwise dimension, all r's must");
      printf(" be included, and if you\n\tare interested in the ");
      printf("behavior at small r's, all points must be\n ");
      printf("\tincluded in the sum.\n");
      exit(-1);
   }

   /* Get rid of trailing spaces on header strings */

   i = strlen(header.name) - 1;
   while(header.name[i] == ' ')
      header.name[i--] = 0;

   i = strlen(header.pars) - 1;
   while(header.pars[i] == ' ')
      header.pars[i--] = 0;

   i = strlen(header.inits) - 1;
   while(header.inits[i] == ' ')
      header.inits[i--] = 0;

}/* check_args */


void error(error_type, error_buf)
char error_type;
char *error_buf;
/* Displays error messages and exits program if fatal */
{
   switch(error_type)
   {
      case COMMAND_LINE_ERROR :
      case INVALID_OPTION :
	 if (error_type == COMMAND_LINE_ERROR)
	    printf("Invalid Command Line. \n");
	 if (error_type == INVALID_OPTION)
	    printf("Invalid Option on Command Line. \n");
	 printf("Usage : grass -M:num_of_reference_pts -E:expansion_factor");
	 printf(" -R:max_distance\n");
	 printf("        input_file_name  output_file_name  min_dimension  ");
	 printf("max_dimension\n");
	 printf("        time_delay  autocorrelation_length  trace[0-2]\n");
	 exit(-1);
	 break;
      case M_OPTION_ERROR :
	 printf("\tError Reading -M Option Value. ");
	 printf("\tCorrect format is -M:num_of_ref_pts\n");
	 exit(-1);
      case E_OPTION_ERROR :
	 printf("\tError Reading -E Option Value. ");
	 printf("\tCorrect format is -E:expansion_factor\n");
	 exit(-1);
      case R_OPTION_ERROR :
	 printf("\tError Reading -R Option Value. ");
	 printf("\tCorrect format is -R:max_dist\n");
	 exit(-1);
      case FILE_ACCESS_ERROR :
	 printf("\tCannot access file %s.\n", error_buf);
	 exit(-1);
	 break;
      case OPEN_OUTPUT_FILE_ERROR :
	 printf("\tCannot Open writable file named %s", error_buf);
	 exit(-1);
      case FILE_FORMAT_ERROR :
	 printf("\t%s is not in the correct format.  Use createheader ");
	 printf("and rerun.\n", error_buf);
	 exit(-1);
      case NOT_SCALAR_DATA :
	 printf("\tThis Programs requires scalar data.\n");
	 exit(-1);
      case ALLOCATION_ERROR :
	 printf("\tFatal error during allocation of %s\n", error_buf);
	 exit(-1);
   }
}/* error */

