#include <math.h>
#include <stdio.h>
#include <string.h>
#include"grass.h"

void read_in_data()
{
   long int i;
   char error_msg;
   FILE *fd;

   /********* Allocate memory for data vector *********/
   data = dvector((long int)1, header.num_pts, &error_msg);
   if (error_msg == ALLOCATION_ERROR)
   {
      printf("\n\tAllocation Failure for data vector.\n");
      exit(-1);
   }

   /****** Reads in data from file ******/
   fd = fopen(input_file_name, "rb");
   fseek(fd, HEADER_LENGTH, 0);

   for (i = 1; i <= header.num_pts; i++)
   {
      if (fread(&data[i], sizeof(double), 1, fd) == 0)
      {
	 printf("The contents of the file does not jive with the header.\n");
	 printf("Try running createheader on the data set.\n");
	 exit(-1);
      }
   }

   fclose(fd);

}/* read_in_data */

void compute_full_correlation_integral()
{
   double 	delta;			/* Interval between ref pts */
   double 	N_float, M_float;
   long int 	next_ref_pt;
   double 	inf_norm, dist;
   long int 	inf_norm_exp;
   double      *m_ptr, *n_ptr;
   long int 	m,n,d, inc_val;
   long int 	i;			/* Used in INT_POW */
   char 	error_msg;
   double 	tmp;


   /** Allocate and zero bin_count matrix **/

   bin_count = imatrix(min_dim, max_dim, (long)0, (long)2048, &error_msg);
   if (error_msg == ALLOCATION_ERROR)
      error(ALLOCATION_ERROR, "bin_count matrix");

   for (d = min_dim; d <= max_dim; d++)
      for (n = 0; n <= 2048; n++)
	 bin_count[d][n] = 0;


   num_autocor_pts = 0;
   delta = (num_embedded_pts - 1.0)/(num_ref_pts - 1.0);
   M_float = 1.5;
   m = M_float;

   if (trace >= 2) printf("\n");
   do
   {
      if (trace >= 2)
	 printf("\rPercentage Completed : %5.2f",100.0*m/num_embedded_pts);

      N_float = 1.5;
      next_ref_pt = N_float;
      for (n = 1; n <= num_embedded_pts; n++)
      {
	 if (n == next_ref_pt)
	 {
	    next_ref_pt = (N_float += delta);
	    if (n <= m)
	       /** We'll look at this pair next time they pop up **/
	       continue;
	    else
	       inc_val = 2;
	 }
	 else
	    inc_val = 1;

	 if (abs(n - m) <= autocor_len)
	    num_autocor_pts += inc_val;
	 else
	 {
	    /*** These points are not autocorrelated ***/
	    inf_norm = 0.0;
	    for (d = 1; d < min_dim; d++)
	    {
	       if (d == 1)
	       {
		  m_ptr = &data[m];
		  n_ptr = &data[n];
	       }
	       else
	       {
		  m_ptr += time_delay;
		  n_ptr += time_delay;
	       }
	       dist = fabs(*m_ptr - *n_ptr);
	       if (dist > inf_norm)
		  inf_norm = dist;
	    }

	    for (d = min_dim; d <= max_dim; d++)
	    {
	       if (d == 1)
	       {
		  m_ptr = &data[m];
		  n_ptr = &data[n];
	       }
	       else
	       {
		  m_ptr += time_delay;
		  n_ptr += time_delay;
	       }
	       dist = fabs(*m_ptr - *n_ptr);
	       if (dist > inf_norm)
	       {
		  inf_norm = dist;
		  INT_POW(inf_norm, expansion_fact, tmp);
		  GET_BIASED_EXP(tmp, inf_norm_exp);
	       }
	       else if (d == min_dim)
	       {
		  INT_POW(inf_norm, expansion_fact, tmp);
		  GET_BIASED_EXP(tmp, inf_norm_exp);
	       }

	       bin_count[d][inf_norm_exp] += inc_val;
	    }/* End for statement through d values */
	 }/* End non-autocorrelated case */
      }/* End for statement through points (n sum) */
   } while ((m = (M_float += delta)) <= num_embedded_pts);
   if (trace >= 2) printf("\n");

}/* compute_correlation_integral */


void compute_small_r_correlation_integral()
{
   long  	i, m, n, d;
   long  	num_bins, bin_value;
   long  	num_pts_completed;
   long  	m_bin, n_bin;
   char 	error_msg;
   char 	error_buf[40];
   long        *point_num;
   double 	inf_norm, dist;
   long  	inf_norm_exp;
   double      *m_ptr, *n_ptr;
   double 	tmp;

   if (trace >= 1)
      printf("Determining Bin Structure ...\n");

   /**** Determine range of data ****/
   max_val = -1e200;
   min_val = 1e200;

   for (i = 1; i <= num_embedded_pts; i++)
   {
      if (data[i] > max_val) max_val = data[i];
      if (data[i] < min_val) min_val = data[i];
   }
   if (trace >= 2)
      printf("\nMaximum_data_value = %lg    Minimum Data Value = %lg\n",
		max_val, min_val);

   if (max_dist > (max_val - min_val))
   {
      printf("The maximum distance entered (%lg) is larger than the size",
	      max_dist);
      printf("of the \nattractor.  Try rerunning with a smaller value.\n");
      exit(-1);
   }

   num_bins = ceil((max_val - min_val)/max_dist);
   if (trace >= 2)
      printf("Number of bins : %ld\n", num_bins);

   /** Allocate and initialize bin structure **/
   bin = binvector((long)1, num_bins, &error_msg);
   if (error_msg == ALLOCATION_ERROR)
      error(ALLOCATION_ERROR, "bin Vector");

   for (i = 1; i <= num_bins; i++)
      bin[i].num_pts = 0;

   /*** Determine Number of Points in each bin ***/
   for (i = 1; i <= num_embedded_pts; i++)
   {
      bin_value = floor((data[i] - min_val)/max_dist) + 1;
      bin[bin_value].num_pts++;
   }

   /*** Allocate memory for the pointers ***/
   for (i = 1; i <= num_bins; i++)
   {
      if (bin[i].num_pts == 0) continue;
      bin[i].point = ivector((long)1, bin[i].num_pts, &error_msg);
      if (error_msg == ALLOCATION_ERROR)
      {
	 sprintf(error_buf, "point vector for bin #%ld", i);
	 error(ALLOCATION_ERROR, error_buf);
      }
   }

   point_num = ivector((long)1, num_bins, &error_msg);
   if (error_msg == ALLOCATION_ERROR)
      error(ALLOCATION_ERROR, "point_num Vector");
   for (i = 1; i <= num_bins; i++)
      point_num[i] = 0;

   for (i = 1; i<= num_embedded_pts; i++)
   {
      bin_value = floor((data[i] - min_val)/max_dist) + 1;
      point_num[bin_value]++;
      bin[bin_value].point[point_num[bin_value]] = i;
   }

   /** Allocate and zero bin_count matrix **/

   bin_count = imatrix(min_dim, max_dim, (long)0, (long)2048, &error_msg);
   if (error_msg == ALLOCATION_ERROR)
      error(ALLOCATION_ERROR, "bin_count matrix");

   for (d = min_dim; d <= max_dim; d++)
      for (n = 0; n <= 2048; n++)
	 bin_count[d][n] = 0;

   if (trace >= 1)
      printf("\nComputing Correlation Integral ...\n");

   num_autocor_pts = 0;
   num_pts_completed = 0;
   for (m_bin = 1; m_bin <= num_bins; m_bin++)
   {
      for (m = 1; m <= bin[m_bin].num_pts; m++)
      {

	 if (trace >= 2)
	 {
	    num_pts_completed++;
	    printf("\rPercentage Completed : %5.2f",
			100.0*num_pts_completed/num_embedded_pts);
	 }

	 for (n_bin = m_bin - 1; n_bin <= m_bin + 1; n_bin++)
	 {
	    if (n_bin < 1 || n_bin > num_bins)
	       continue;
	    for (n = 1; n <= bin[n_bin].num_pts; n++)
	    {
	       if (bin[n_bin].point[n] <= bin[m_bin].point[m])
		  /** We'll get this pair next time they come through **/
		  continue;

	       if (abs(data[bin[m_bin].point[m]] - data[bin[n_bin].point[n]])
		   <= autocor_len)
		  num_autocor_pts += 2;
	       {
		  inf_norm = 0.0;
		  for (d = 1; d < min_dim; d++)
		  {
		     if (d == 1)
		     {
			m_ptr = &data[bin[m_bin].point[m]];
			n_ptr = &data[bin[n_bin].point[n]];
		     }
		     else
		     {
			m_ptr += time_delay;
			n_ptr += time_delay;
		     }
		     dist = fabs(*m_ptr - *n_ptr);
		     if (dist > inf_norm)
			inf_norm = dist;
		  }

		  for (d = min_dim; d <= max_dim; d++)
		  {
		     if (d == 1)
		     {
			m_ptr = &data[bin[m_bin].point[m]];
			n_ptr = &data[bin[n_bin].point[n]];
		     }
		     else
		     {
			m_ptr += time_delay;
			n_ptr += time_delay;
		     }

		     dist = fabs(*m_ptr - *n_ptr);
		     if (dist > inf_norm)
		     {
			inf_norm = dist;
			INT_POW(inf_norm, expansion_fact, tmp);
			GET_BIASED_EXP(tmp, inf_norm_exp);
		     }
		     else if (d == min_dim)
		     {
			INT_POW(inf_norm, expansion_fact, tmp);
			GET_BIASED_EXP(tmp, inf_norm_exp);
		     }

		     bin_count[d][inf_norm_exp] += 2;
		  }/* End for statement through d values */
	       }/* End non-autocorrelated case */
	    }/* End for statement through points in one bin(n sum) */
	 } /* End for statement through bins in n sum */
      }/* End for statement through points in one bin of m sum */
   }/* End for statement through bins in m sum */

   if (trace >= 2) printf("\n");

   free_ivector(point_num, (long)1, num_bins);

} /* compute_small_r_correlation_integral */


void write_output()
{
   long  	min_r, max_r;
   double 	log_of_r, log_of_2;
   double 	log_of_C, min_log_of_C;
   double 	min_dist_found, max_dist_found;
   long  	i, d;
   long        *C;
   FILE        *fd;
   char 	stop, error_msg;
   double 	tmp;

   fd = fopen(output_file_name,"w");

   log_of_2 = log10(2.0);
   min_log_of_C = log10(1.0/(num_ref_pts*num_embedded_pts));


   /**** Find minimum and maximum distances found ***/
   i = 1;
   stop = FALSE;
   do
   {
      for (d = min_dim; d <= max_dim; d++)
      {
	 if (bin_count[d][i] != 0)
	 {
	    min_r = i;
	    stop = TRUE;
	    break;
	 }
      }
      i++;
   }while (!stop);

   if(COMPUTE_FULL_CORR_INT == TRUE)
   {
      i = 2048;
      stop = FALSE;
      do
      {
	 for (d = min_dim; d <= max_dim; d++)
	 {
	    if (bin_count[d][i] != 0)
	    {
	       max_r = i;
	       stop = TRUE;
	       break;
	    }
	 }
	 i--;
      }while (!stop);
   }
   else
   {
      INT_POW(max_dist, expansion_fact, tmp);
      GET_BIASED_EXP(tmp, max_r);
   }

   if (trace >= 2)
   {

      min_dist_found = pow(2.0 ,((min_r - 1023.0)/expansion_fact));
      printf("minimum distance found : %g\n", min_dist_found);
      max_dist_found = pow(2.0, ((max_r - 1023.0)/expansion_fact));
      printf("maximum distance found : %g\n", max_dist_found);
      printf("Number of Points on Graph : %ld\n", max_r - min_r + 1);
   }

   /*** Annotate the beginning of the data file ***/

   fprintf(fd,"#GRASS Output Results\n");
   fprintf(fd,"#Information regarding %s's Contents : \n",
		input_file_name);
   fprintf(fd,"#Attractor Name : %s\n", header.name);
   fprintf(fd,"#Attractor Parameters : %s\n", header.pars);
   fprintf(fd,"#Number of Dimensions of Attractor : %d\n", header.num_dim);
   fprintf(fd,"#Initial Values : %s\n", header.inits);
   fprintf(fd,"#Number of points in Data file according to header : %ld\n"
	   ,header.num_pts);
   fprintf(fd,"#Number of points in transient : %ld\n"
	   ,header.tran_len);
   fprintf(fd,"#Execution Parameters : \n");
   fprintf(fd,"#Range of Embedding Dimensions : %ld to %ld\n", min_dim,
	     max_dim);
   fprintf(fd,"#Time Delay Value : %ld\n", time_delay);
   fprintf(fd,"#Autocorrelation Length : %ld\n", autocor_len);
   fprintf(fd,"#Number of points in largest embedding space : %ld\n",
	   num_embedded_pts);
   if (INPUT_NUM_REF_POINTS == TRUE)
   {
      fprintf(fd,"#Computing Averaged Pointwise dimension with %ld reference"
	  ,num_ref_pts);
      fprintf(fd,"points \n#used in calculation\n");
   }
   fprintf(fd,"#Expansion Factor : %ld\n", expansion_fact);
   if (COMPUTE_FULL_CORR_INT == FALSE)
      fprintf(fd,"#Computing Correlation Integral up to r = %lg\n",
	     max_dist);

   fprintf(fd, "#Minimum distance found : %g\n", min_dist_found);
   fprintf(fd, "#Maximum distance found : %g\n", max_dist_found);
   fprintf(fd, "#Number of Points on Graph : %ld\n", max_r - min_r + 1);

   C = ivector(min_dim, max_dim, &error_msg);
   if (error_msg == ALLOCATION_ERROR)
      error(ALLOCATION_ERROR, "C vector");

   for (d = min_dim; d <= max_dim; d++)
      C[d] = 0;

   for (i = min_r; i <= max_r; i++)
   {
      log_of_r = ((i - 1023.0)/expansion_fact) * log_of_2;
      fprintf(fd, "%f ", log_of_r);

      for (d = min_dim; d <= max_dim; d++)
      {
	 C[d] += bin_count[d][i];
	 if (C[d] != 0)
	 {
	    log_of_C = log10((double)C[d]/
		   (num_ref_pts * (num_embedded_pts - 1) - num_autocor_pts));
	    fprintf(fd, "%f ", log_of_C);
	 }
	 else
	    fprintf(fd, "%f ", min_log_of_C);
      }
      fprintf(fd, "\n");
   }
   fclose(fd);

printf("C = %ld\n", C[max_dim]);
printf("num_embedded_pts = %ld\n", num_embedded_pts);
printf("num_ref_pts = %ld\n", num_ref_pts);
printf("num_autocor_pts = %ld\n", num_autocor_pts);
   free_ivector(C, min_dim, max_dim);

}/* write_output */


void determine_number_of_points()
{
   num_embedded_pts = header.num_pts - time_delay*(max_dim - 1);

   if (INPUT_NUM_REF_POINTS == FALSE)
      num_ref_pts = num_embedded_pts;

}/* determine_number_of_points */
