/****************************************************************/
/*								*/
/*   This program reads a set of points from an existing file   */
/*   and creates an output file which contains appropriate an   */
/*   header in the format to be used by gendim.                 */
/*								*/
/****************************************************************/

#define VARS

#include <stdio.h>
#include "../include/header.h"
#include "../include/getinput.h"

#define TRUE 1
#define FALSE 0

void main()
{
   char file_name[NAME_LENGTH];
   char output_file_name[NAME_LENGTH];
   char data_name[NAME_LENGTH];
   char description[NAME_LENGTH];
   char init_buf[NAME_LENGTH];
   char buf[80];
   char *buf_ptr;
   long int num_pts;
   long int tran_len;
   long int num_dims;
   char dim_names[NAME_LENGTH];
   long int format;
   long int i, n;
   char input, stop;
   double data_val;
   float float_data_val;
   FILE *fd, *fout;

   get_input("Enter the name of the data file : ", file_name, NAME);

   /*** Verify existence of file ***/

   if ((fd = fopen(file_name, "r")) == NULL)
   {
      printf("Cannot open %s.\n", file_name);
      exit(-1);
   }

   printf("\nThe file header will contain the following information :\n");
   printf("\t1. Data Name\n");
   printf("\t2. Short description\n");
   printf("\t3. Initial Conditions\n");
   printf("\t4. Name and number of dimension\n");
   printf("\t5. Number of points in set and transient\n\n");

   get_input("Enter Data Name : ", data_name, STRING);
   if (data_name[0] == 0) sprintf(data_name, "None");

   sprintf(buf, "Enter short description (%d characters or less) :\n",
	   NAME_LENGTH);
   get_input(buf, description, STRING);
   if (description[0] == 0) sprintf(description, "None");

   sprintf(buf, "Enter the initial conditions (%d characters or less) :\n",
	   NAME_LENGTH);
   get_input(buf, init_buf, STRING);
   if (init_buf[0] == 0) sprintf(init_buf, "None");

   get_input("Enter number of points in transient : ", &tran_len, POS_INT);

   get_input("Enter the number of dimensions in data set : ", &num_dims,
	      POS_INT);

   for (i = 1; i <= num_dims; i++)
   {
      sprintf(buf, "Enter one character name of dimension %d : ", i);
      get_input(buf, &dim_names[i - 1], CHAR);
   }
   dim_names[num_dims] = 0;

   printf("\nWhat is the format of the data file :\n");
   printf("\t1. Ascii tables \n");
   printf("\t2. Binary floats \n");
   printf("\t3. Binary double \n");

   input = OK;
   do
   {
      get_input("Enter 1-3 : ", &format, POS_INT);
      if (format < 1 || format > 3)
      {
	 printf("\tInvalid Entry.  Try again. \n\n");
	 input = NOT_OK;
      }
   } while (input == NOT_OK);

   get_input("Enter the name of the output file : ", output_file_name,
	      NAME);

   /*** Open output file ***/

   if ((fout = fopen(output_file_name, "wb")) == NULL)
   {
      printf("Cannot open writable file named %s.\n",
	      output_file_name);
      exit(-1);
   }

   write_header(fout, SYS_TYPE, data_name, description, init_buf,
		(long int) 0, tran_len, num_dims, dim_names);

   num_pts = 0;

   switch (format)
   {
      case 1 :
	 /* Ascii file */
	 get_input("Enter Number of points in file : ", &num_pts, POS_INT);

	 for(i = 1; i <= num_pts; i++)
	 {
	    for(n = 1; n <= num_dims; n++)
	    {
	       fscanf(fd, "%lf", &data_val);
	       fwrite(&data_val, sizeof(double), 1, fout);
	    }
	 }
	 break;
      case 2 :
	 stop = FALSE;
	 do
	 {
	    if (fread(&float_data_val, sizeof(float), 1, fd) == 0)
	       stop = TRUE;
	    else
	    {
	       data_val = (double)float_data_val;
	       fwrite(&data_val, sizeof(double), 1, fout);
	       num_pts++;
	    }
	 } while (!stop);
	 num_pts /= num_dims;
	 printf("%ld points read into file %s\n", num_pts, output_file_name);
	 break;
      case 3 :
	 /* Binary doubles */
	 stop = FALSE;
	 do
	 {
	    if (fread(&data_val, sizeof(double), 1, fd) == 0)
	       stop = TRUE;
	    else
	    {
	       fwrite(&data_val, sizeof(double), 1, fout);
	       num_pts++;
	    }
	 } while (!stop);
	 num_pts /= num_dims;
	 printf("%ld points read into file %s\n", num_pts, output_file_name);
	 break;
   } /* End switch statement */


   /* write number of points into header. */
   fseek(fout, NUM_PTS_OFFSET, 0);
   fwrite(&num_pts, 4, 1, fout);

   fclose(fd);
   fclose(fout);
}
