/****************************************************************/
/*                                                              */
/*      This program reads the header of the inputed file       */
/*      and displays its vital statistics, including minimum    */
/*      and maximum values 					*/
/*                                                              */
/*      Usage : whatsin [file_name1] [file_name1] ...           */
/*								*/
/*								*/
/****************************************************************/

#define VARS

#include <stdio.h>
#include <math.h>
#include "../include/header.h"

#define ALLOCATION_ERROR 0
#define TRUE 1
#define FALSE 0

double *dvector();
void free_dvector();

void main(argc, argv)
int argc;
char *argv[];
{
   int n, i, j;
   char buf[80];
   int offset;
   int display_num;
   double val;
   double norm, rad;
   FILE *fd;
   double *min, *max, *avg;
   char error_msg;

   if (argc == 1)
   {
      printf("Usage : whatisin [file_name1] [file_name2] ... \n");
      exit(-1);
   }

   for (n = 1; n < argc; n++)
   {
      /* Open File and read header */

      if ((fd = fopen(argv[n],"rb")) == NULL)
      {
         printf("\nError Opening File %s. \n\n", argv[n]);
         continue;
      }

      if (read_header(fd) == FILE_FORMAT_ERROR)
      {
         printf("\nFile %s is not in the correct format.\n\n", argv[n]);
         continue;
      }

      /* Concatenate strings */
      i = strlen(header.name) - 1;
      while(header.name[i] == ' ')
         header.name[i--] = 0;

      i = strlen(header.pars) - 1;
      while(header.pars[i] == ' ')
         header.pars[i--] = 0;

      i = strlen(header.inits) - 1;
      while(header.inits[i] == ' ')
         header.inits[i--] = 0;

      /* Print contents of header to screen */
      printf("\n\tContents of %s :\n\n", argv[n]);
      printf("Name : %s\n", header.name);
      printf("Data parameters : %s\n",header.pars);
      strcpy(buf, "Attractor Dimensions : ");
      offset = 23;
      for (i = 0; i < header.num_dim; i++)
      {
         buf[offset+i] = header.dim_names[i];
         buf[offset+i+1] = 0;
         if (i < (header.num_dim - 2))
         {
            strcat(buf, ", ");
            offset+=2;
         }
         else if (i == header.num_dim - 2)
         {
            strcat(buf, ", and ");
            offset+= 6;
         }
         else
            strcat(buf,".");
      }
      printf("%s\n",buf);
      printf("Initial Values : %s\n", header.inits);
      printf("Number of Data Points : %ld\n", header.num_pts);
      printf("Length of transient : %ld\n", header.tran_len);
      if (header.sys_type == DOS)
         strcpy(buf, "Dos");
      else
         strcpy(buf, "Unix");
      printf("File created on %s machine.\n\n", buf);

      /* Display first five entries of file */
      display_num=(header.num_pts > 5 ? 5 : header.num_pts);
      printf("First %d data entries\n", display_num);
      printf("Entry #");
      for (i = 0; i < header.num_dim; i++)
	 printf("\t   %c", header.dim_names[i]);
      printf("\n");
      for (i = 1; i <= display_num; i++)
      {
	 printf("  %d", i);
	 for (j = 0; j < header.num_dim; j++)
	 {
	    fread(&val, 8, 1, fd);
	    printf("\t%6.3lg", val);
	 }
	 printf("\n");
      }

      /* Find Minimum and Maximum of Data */
      fseek(fd, HEADER_LENGTH, 0);

      min = dvector((long)1, header.num_dim, &error_msg);
      max = dvector((long)1, header.num_dim, &error_msg);
      avg = dvector((long)1, header.num_dim, &error_msg);
      for (i = 1; i <= header.num_dim; i++)
      {
         min[i] = 1e90;
         max[i] = -1e90;
         avg[i] = 0;
      }
      for (i = 1; i <= header.num_pts; i++)
      {
         for(j = 1; j <= header.num_dim; j++)
 	 {
  	    fread(&val, sizeof(double), 1, fd);
   	    if (val > max[j])  max[j] = val;
   	    if (val < min[j])  min[j] = val;
	    avg[j] += val;
         }
      }
      printf("\nData Statistics \n");
      for (i = 0; i < header.num_dim; i++)
	 printf("\t   %c", header.dim_names[i]);
      printf("\n");
      printf("Minimum");
      for (i = 1; i <= header.num_dim; i++)
	 printf("\t%6.3lg", min[i]);
      printf("\nMaximum");
      for (i = 1; i <= header.num_dim; i++)
	 printf("\t%6.3lg", max[i]);
      printf("\n");
      
         for (i = 1; i <= header.num_dim; i++)
	    avg[i] /= header.num_pts;
	 rad = 0.0;

         fseek(fd, HEADER_LENGTH, 0);
 
         for (i = 1; i <= header.num_pts; i++)
         {  
	    norm = 0.0;
            for(j = 1; j <= header.num_dim; j++)
            {
               fread(&val, sizeof(double), 1, fd);
	       if (fabs(val - avg[j]) > norm)
		  norm = fabs(val - avg[j]);
            }
	    rad += norm;
         }  
         rad /= header.num_pts;
         printf("Radius of Attractor :  %f.\n\n", rad);
      
      fclose(fd);
   }
}


double *dvector(nl,nh, error_msg)
long int nl,nh;
char *error_msg;
{
   double *v;

   v=(double *)malloc((unsigned) (nh-nl+1)*sizeof(double));
   if (!v)
   {
      *error_msg = ALLOCATION_ERROR;
      return(v);
   }
   *error_msg = OK;
   return v-nl;
}/* dvector */


void free_dvector(v,nl,nh)
double *v;
long int nl,nh;
{
   free((char*) (v+nl));
}/* free_dvector */

